from __future__ import annotations

import numpy as np
import pygame
from loguru import logger as log

# todo: add pygame, websockets to requirements.txt


class PygameKeyboardClient:
    def __init__(self, width=640, height=480, title="Keyboard Control"):
        pygame.init()
        self.screen = pygame.display.set_mode((width, height))
        pygame.display.set_caption(title)

        self.pressed_keys = set()

        self.instructions = [
            "===== Keyboard Controls Instructions =====",
            "support combination of multiple keys",
            "example: press UP+RIGHT for diagonal move",
            "",
            "   Movement (world EE coords):",
            "     UP    - Move +X",
            "     DOWN  - Move -X",
            "     LEFT  - Move +Y",
            "     RIGHT - Move -Y",
            "     e     - Move +Z",
            "     d     - Move -Z",
            "",
            "   Rotation (local EE coords):",
            "     q     - Roll + ",
            "     w     - Roll - ",
            "     a     - Pitch + ",
            "     s     - Pitch - ",
            "     z     - Yaw + ",
            "     x     - Yaw - ",
            "",
            "   Gripper:",
            "     Space - Close(hold) / Open(release)",
            "",
            "     ESC   - Quit Simulation",
            "==========================================",
        ]

    def update(self) -> bool:
        """
        fresh pygame event, update pressed_keys.
        """
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return False
            elif event.type == pygame.KEYDOWN:
                self.pressed_keys.add(event.key)
            elif event.type == pygame.KEYUP:
                if event.key in self.pressed_keys:
                    self.pressed_keys.remove(event.key)
        return True

    def is_pressed(self, key: int) -> bool:
        return (key in self.pressed_keys) or pygame.key.get_pressed()[key]

    def close(self):
        pygame.quit()

    def draw_instructions(self):
        font = pygame.font.Font(pygame.font.match_font("DejaVu Sans Mono"), 25)  # monospaced font Courier
        text_color = (255, 255, 255)
        bg_color = (0, 0, 0)

        self.screen.fill(bg_color)

        y_offset = 20
        for line in self.instructions:
            text_surface = font.render(line, True, text_color)
            self.screen.blit(text_surface, (20, y_offset))
            y_offset += 30

        pygame.display.flip()


def process_kb_input(
    keyboard_client: PygameKeyboardClient, dpos: float, drot: float
) -> tuple[np.ndarray, np.ndarray, bool]:
    delta_pos = np.zeros(3, dtype=np.float32)
    delta_rot = np.zeros(3, dtype=np.float32)

    key_to_action = {
        pygame.K_UP: (delta_pos, np.array([dpos, 0.0, 0.0], dtype=np.float32)),
        pygame.K_DOWN: (delta_pos, np.array([-dpos, 0.0, 0.0], dtype=np.float32)),
        pygame.K_LEFT: (delta_pos, np.array([0.0, dpos, 0.0], dtype=np.float32)),
        pygame.K_RIGHT: (delta_pos, np.array([0.0, -dpos, 0.0], dtype=np.float32)),
        pygame.K_e: (delta_pos, np.array([0.0, 0.0, dpos], dtype=np.float32)),
        pygame.K_d: (delta_pos, np.array([0.0, 0.0, -dpos], dtype=np.float32)),
        pygame.K_q: (delta_rot, np.array([drot, 0.0, 0.0], dtype=np.float32)),
        pygame.K_w: (delta_rot, np.array([-drot, 0.0, 0.0], dtype=np.float32)),
        pygame.K_a: (delta_rot, np.array([0.0, drot, 0.0], dtype=np.float32)),
        pygame.K_s: (delta_rot, np.array([0.0, -drot, 0.0], dtype=np.float32)),
        pygame.K_z: (delta_rot, np.array([0.0, 0.0, drot], dtype=np.float32)),
        pygame.K_x: (delta_rot, np.array([0.0, 0.0, -drot], dtype=np.float32)),
    }

    # cache pressed keys first rather query keyboard_client.is_pressed(key) every time to save computation
    pressed_keys = {key: keyboard_client.is_pressed(key) for key in key_to_action.keys()}
    for key, (target, value) in key_to_action.items():
        if pressed_keys.get(key):
            target += value

    close_gripper = keyboard_client.is_pressed(pygame.K_SPACE)

    return delta_pos, delta_rot, close_gripper


#############################################################
################### android phone
#############################################################

import asyncio
import json
import threading
import time

import websockets


class PhoneServer:
    def __init__(self, translation_step, host="0.0.0.0", port=8765, update_dt=1 / 50):
        self.host = host
        self.port = port
        self.update_dt = update_dt
        self.translation_step = translation_step

        self._lock = threading.Lock()

        self._latest_data = {
            "buttonStates": [False] * 6,
            "isSwitch1On": False,
            "isSwitch2On": False,
            "rotation": [0.0, 0.0, 0.0, 1.0],
        }

        self.q_world = np.array([0.0, 0.0, 0.0, 1.0], dtype=np.float64)
        self.delta_position = np.zeros(3)
        self.rotation_flag = False
        self.gripper_flag = False

    def key_map(self, data):
        delta_position = np.zeros(3)

        button_states = data["buttonStates"]
        isSwitch1On = data["isSwitch1On"]
        isSwitch2On = data["isSwitch2On"]

        rotation_flag = True if isSwitch1On else False
        gripper_flag = True if isSwitch2On else False

        # kep map
        if button_states[0]:
            delta_position[2] = -self.translation_step
        if button_states[1]:
            delta_position[2] = self.translation_step
        if button_states[2]:  # todo: check why the y axis is inversed
            delta_position[1] = -self.translation_step
        if button_states[5]:
            delta_position[1] = self.translation_step
        if button_states[4]:
            delta_position[0] = -self.translation_step
        if button_states[3]:
            delta_position[0] = self.translation_step

        q_world = data.get("rotation", [0.0, 0.0, 0.0, 1.0])[:4]
        q_world = np.array(q_world, dtype=np.float64)

        return q_world, delta_position, gripper_flag

    async def handler(self, websocket, path):
        """
        whenever a client connects, this handler will be called.
        upon receiving data, update self._latest_data.
        """

        log.info("A client connected.")
        time.sleep(3)
        self.receive_flag = True

        while True:
            try:
                message = await websocket.recv()
                # print("Received from client:", message)

                data = json.loads(message)

                q_world, delta_position, gripper_flag = self.key_map(data)

                # update latest data (thread-safe)
                with self._lock:
                    self.q_world = q_world
                    self.delta_position = delta_position
                    self.gripper_flag = gripper_flag

            except websockets.ConnectionClosed:
                self.receive_flag = False
                log.info("Connection closed.")
                break
            except Exception as e:
                self.receive_flag = False
                log.error(f"Error in handler: {e}")
                break
            except KeyboardInterrupt:
                self.receive_flag = False
                log.info("KeyboardInterrupt.")
                break

    async def main_server(self):
        """
        start the websockets server (async)
        """
        async with websockets.serve(self.handler, self.host, self.port):
            log.info(f"WebSocket Server started, listening on {self.host}:{self.port}")
            await asyncio.Future()  # run forever

    def start_server(self):
        """
        start the server in a new thread
        """

        def run_asyncio():
            asyncio.run(self.main_server())

        server_thread = threading.Thread(target=run_asyncio, daemon=True)
        server_thread.start()

    def get_latest_data(self):
        """
        get the latest position, orientation (thread-safe).
        return: (position: np.ndarray, orientation: np.ndarray)
        """
        with self._lock:
            # pos = self._latest_data["position"].copy()
            q_world = self.q_world.copy()
            delta_position = self.delta_position.copy()
            gripper_flag = self.gripper_flag
        return q_world, delta_position, gripper_flag

    @property
    def _latest_position(self):
        """
        internal attribute: get the latest position (no lock, for internal use only)
        """
        return self._latest_data["position"]


def quaternion_inverse(q):
    norm_q = np.linalg.norm(q)
    if norm_q < 1e-12:
        log.warning("Warning: Quaternion norm is too small, potential divide by zero.")
        return np.array([0, 0, 0, 1])  # Return a default unit quaternion
    q_inv = q * np.array([1, -1, -1, -1])
    return q_inv / norm_q


def quaternion_multiply(q1, q2):
    w1, x1, y1, z1 = q1
    w2, x2, y2, z2 = q2
    return np.array([
        w1 * w2 - x1 * x2 - y1 * y2 - z1 * z2,
        w1 * x2 + x1 * w2 + y1 * z2 - z1 * y2,
        w1 * y2 - x1 * z2 + y1 * w2 + z1 * x2,
        w1 * z2 + x1 * y2 - y1 * x2 + z1 * w2,
    ])


def get_delta_quaternion(quaternion1, quaternion2):
    """delta q = q2 * q1^-1"""
    q1_inv = quaternion_inverse(quaternion1)
    delta_q = quaternion_multiply(quaternion2, q1_inv)
    return delta_q


def quaternion_conjugate(q):
    w, x, y, z = q
    return np.array([w, -x, -y, -z])


def rotate_vector_with_quaternion(q, v):
    v_quaternion = np.array([0, v[0], v[1], v[2]])  # 将向量 v 转化为四元数
    q_conjugate = quaternion_conjugate(q)
    # v' = q * v * q^-1
    rotated_quaternion = quaternion_multiply(quaternion_multiply(q, v_quaternion), q_conjugate)
    return rotated_quaternion[1:]


TRANSFORMATION_MATRIX = np.array([[0, 0, 1], [0, 1, 0], [-1, 0, 0]])


def transform_position(joystick_coords):
    transformed_position = np.dot(TRANSFORMATION_MATRIX, joystick_coords)
    return transformed_position


def transform_orientation(quaternion_orientation):
    rotation_quaternion = rotation_matrix_to_quaternion(TRANSFORMATION_MATRIX)
    transformed_quaternion = quaternion_multiply(
        quaternion_multiply(rotation_quaternion, quaternion_orientation), quaternion_conjugate(rotation_quaternion)
    )
    return transformed_quaternion


def rotation_matrix_to_quaternion(R):
    trace = np.trace(R)
    if trace > 0:
        s = 0.5 / np.sqrt(trace + 1.0)
        w = 0.25 / s
        x = (R[2, 1] - R[1, 2]) * s
        y = (R[0, 2] - R[2, 0]) * s
        z = (R[1, 0] - R[0, 1]) * s
    else:
        if R[0, 0] > R[1, 1] and R[0, 0] > R[2, 2]:
            s = 2.0 * np.sqrt(1.0 + R[0, 0] - R[1, 1] - R[2, 2])
            w = (R[2, 1] - R[1, 2]) / s
            x = 0.25 * s
            y = (R[0, 1] + R[1, 0]) / s
            z = (R[0, 2] + R[2, 0]) / s
        elif R[1, 1] > R[2, 2]:
            s = 2.0 * np.sqrt(1.0 + R[1, 1] - R[0, 0] - R[2, 2])
            w = (R[0, 2] - R[2, 0]) / s
            x = (R[0, 1] + R[1, 0]) / s
            y = 0.25 * s
            z = (R[1, 2] + R[2, 1]) / s
        else:
            s = 2.0 * np.sqrt(1.0 + R[2, 2] - R[0, 0] - R[1, 1])
            w = (R[1, 0] - R[0, 1]) / s
            x = (R[0, 2] + R[2, 0]) / s
            y = (R[1, 2] + R[2, 1]) / s
            z = 0.25 * s
    return np.array([w, x, y, z])


def quaternion_to_rotation_matrix(quaternion):
    qx, qy, qz, qw = quaternion
    norm_q = np.sqrt(qx * qx + qy * qy + qz * qz + qw * qw)
    if norm_q < 1e-12:
        log.warning("Warning: Quaternion norm is too small, potential divide by zero.")
        return np.eye(3)  # Return the identity matrix if the quaternion is invalid
    qx, qy, qz, qw = qx / norm_q, qy / norm_q, qz / norm_q, qw / norm_q
    R = np.array(
        [
            [1 - 2 * (qy**2 + qz**2), 2 * (qx * qy - qz * qw), 2 * (qx * qz + qy * qw)],
            [2 * (qx * qy + qz * qw), 1 - 2 * (qx**2 + qz**2), 2 * (qy * qz - qx * qw)],
            [2 * (qx * qz - qy * qw), 2 * (qy * qz + qx * qw), 1 - 2 * (qx**2 + qy**2)],
        ],
        dtype=np.float64,
    )
    return R



from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class PutGroceriesInCupboardMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/put_groceries_in_cupboard/v2"
    objects = [
        RigidObjMetaCfg(
            name="cupboard",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/cupboard/usd/cupboard.usd",
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="chocolate_jello_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/chocolate_jello_visual/usd/chocolate_jello_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="strawberry_jello_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/strawberry_jello_visual/usd/strawberry_jello_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="spam_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/spam_visual/usd/spam_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="sugar_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/sugar_visual/usd/sugar_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="crackers_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/crackers_visual/usd/crackers_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="mustard_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/mustard_visual/usd/mustard_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="soup_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/soup_visual/usd/soup_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="tuna_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/tuna_visual/usd/tuna_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="coffee_visual",
            filepath="roboverse_data/assets/rlbench/put_groceries_in_cupboard/coffee_visual/usd/coffee_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    # TODO: add checker



# ruff: noqa: F401

import importlib

__CANDIDATE_MODULES = [
    "peg_insertion_side",
    "pick_single_egad",
    "pick_single_ycb",
]


def __getattr__(name):
    from .pick_cube_metacfg import PickCubeMetaCfg
    from .plug_charger_metacfg import PlugChargerMetaCfg
    from .stack_cube_metacfg import StackCubeMetaCfg
    from .stack_green_cube_metacfg import StackGreenCubeMetaCfg

    if name in locals():
        return locals()[name]

    # Lazy load modules
    for module_name in __CANDIDATE_MODULES:
        module = importlib.import_module(f".{module_name}", __name__)
        if name in module.__dict__:
            return getattr(module, name)

    raise AttributeError(f"module {__name__} has no attribute {name}")



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotScoopRightLadle196MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-213438_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle197MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-215322_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle198MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-212105_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle199MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-214009_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle200MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-213523_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle201MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-213043_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightLadle202MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="ladle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ce27e3369587476c85a436905f1e5c00/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ce27e3369587476c85a436905f1e5c00_scoop_right/20240826-211805_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon556MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-220952_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon557MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-222255_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon558MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-221139_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon559MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-222123_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon560MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-215355_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon561MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/031_spoon_google_16k/031_spoon_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/031_scoop_right/20240826-211626_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon919MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-212150_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon920MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-212324_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon921MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-213634_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon922MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-213404_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon923MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-213059_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon924MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-213202_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon925MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/2613c5f24bb3407da2664c410c34d523/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/2613c5f24bb3407da2664c410c34d523_scoop_right/20240826-211336_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1002MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-211621_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1003MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-220138_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1004MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-213947_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1005MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-220849_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1006MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-220735_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1007MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/e1745c4aa7c046a2b5193d2d23be0192/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/e1745c4aa7c046a2b5193d2d23be0192_scoop_right/20240826-220804_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1347MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-213418_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1348MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-213451_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1349MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-220845_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1350MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-220325_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1351MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-212549_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotScoopRightSpoon1352MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="spoon",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/218768789fdd42bd95eb933046698499/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/218768789fdd42bd95eb933046698499_scoop_right/20240826-215203_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from __future__ import annotations

from metasim.utils.configclass import configclass

from .base_scene_metacfg import SceneMetaCfg


@configclass
class TapwaterScene144MetaCfg(SceneMetaCfg):
    """Config class for tapwater scene"""

    name: str = "tapwater_144"
    filepath: str = "roboverse_data/scenes/arnold/144/layout.usd"
    positions: list[tuple[float, float, float]] = [
        (-2.06621, 0.8603, -0.47689),
        (-0.87213, -0.42288, 0.00183),
    ]  # XXX: only positions are randomized for now
    default_position: tuple[float, float, float] = (-2.06621, 0.8603, -0.47689)
    quat: tuple[float, float, float, float] = (0.7071068, 0.7071068, 0.0, 0.0)
    scale: tuple[float, float, float] = (0.01, 0.01, 0.01)



from metasim.cfg.objects import ArticulationObjMetaCfg
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class OpenBoxMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 250
    traj_filepath = "roboverse_data/trajs/rlbench/open_box/v2"
    objects = [
        ArticulationObjMetaCfg(
            name="box_base",
            fix_base_link=True,
            filepath="roboverse_data/assets/rlbench/close_box/box_base/usd/box_base.usd",
            urdf_path="roboverse_data/assets/rlbench/close_box/box_base/urdf/box_base_unique.urdf",
            mjcf_path="roboverse_data/assets/rlbench/close_box/box_base/mjcf/box_base_unique.mjcf",
        ),
    ]
    # TODO: add checker



from metasim.cfg.objects import ArticulationObjMetaCfg
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg

_OBJECTS = [
    ArticulationObjMetaCfg(
        name="drawer_frame",
        filepath="roboverse_data/assets/rlbench/close_drawer/drawer_frame/usd/drawer_frame.usd",
    ),
]


@configclass
class CloseDrawerMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/close_drawer/v2"
    objects = _OBJECTS
    # TODO: add checker


@configclass
class OpenDrawerMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/open_drawer/v2"
    objects = _OBJECTS
    # TODO: add checker



from __future__ import annotations

import logging
import os
import time
from typing import Literal

import imageio as iio
import numpy as np
import rootutils
import tyro
from loguru import logger as log
from numpy.typing import NDArray
from rich.logging import RichHandler
from tyro import MISSING

logging.addLevelName(5, "TRACE")
log.configure(handlers=[{"sink": RichHandler(), "format": "{message}"}])
rootutils.setup_root(__file__, pythonpath=True)

from metasim.cfg.randomization import RandomizationMetaCfg
from metasim.cfg.render import RenderMetaCfg
from metasim.utils import configclass


@configclass
class Args:
    task: str = MISSING
    robot: str = "franka"
    scene: str | None = None
    render: RenderMetaCfg = RenderMetaCfg()
    random: RandomizationMetaCfg = RandomizationMetaCfg()

    ## Handlers
    sim: Literal["isaaclab", "isaacgym", "pyrep", "pybullet", "sapien", "mujoco", "blender"] = "isaaclab"
    renderer: Literal["isaaclab", "isaacgym", "pyrep", "pybullet", "sapien", "mujoco", "blender"] | None = None

    ## Others
    num_envs: int = 1
    try_add_table: bool = True
    object_states: bool = False
    split: Literal["train", "val", "test", "all"] = "all"
    headless: bool = False

    ## Only in args
    save_image_dir: str | None = "tmp"
    save_video_path: str | None = None
    stop_on_runout: bool = False

    def __post_init__(self):
        log.info(f"Args: {self}")


args = tyro.cli(Args)


###########################################################
## Utils
###########################################################
def get_actions(all_actions, action_idx: int, num_envs: int):
    envs_actions = all_actions[:num_envs]
    actions = [
        env_actions[action_idx] if action_idx < len(env_actions) else env_actions[-1] for env_actions in envs_actions
    ]
    return actions


def get_states(all_states, action_idx: int, num_envs: int):
    envs_states = all_states[:num_envs]
    states = [env_states[action_idx] if action_idx < len(env_states) else env_states[-1] for env_states in envs_states]
    return states


def get_runout(all_actions, action_idx: int):
    runout = all([action_idx >= len(all_actions[i]) for i in range(len(all_actions))])
    return runout


class ObsSaver:
    def __init__(self, image_dir: str | None = None, video_path: str | None = None):
        assert image_dir is not None or video_path is not None
        self.image_dir = image_dir
        self.video_path = video_path
        self.images: list[NDArray] = []

        self.image_idx = 0

    def add(self, obs: dict):
        from torchvision.utils import make_grid, save_image

        if obs is None or obs.get("rgb", None) is None:
            return

        rgb_data = obs["rgb"]  # (N, H, W, C), uint8
        image = make_grid(rgb_data.permute(0, 3, 1, 2) / 255, nrow=int(rgb_data.shape[0] ** 0.5))  # (C, H, W)

        if self.image_dir is not None:
            os.makedirs(self.image_dir, exist_ok=True)
            save_image(image, os.path.join(self.image_dir, f"rgb_{self.image_idx:04d}.png"))
            self.image_idx += 1

        image = image.cpu().numpy().transpose(1, 2, 0)  # (H, W, C)
        image = (image * 255).astype(np.uint8)
        self.images.append(image)

    def save(self):
        if self.video_path is not None:
            log.info(f"Saving video of {len(self.images)} frames to {self.video_path}")
            os.makedirs(os.path.dirname(self.video_path), exist_ok=True)
            iio.mimsave(self.video_path, self.images, fps=30)


###########################################################
## Main
###########################################################
def main():
    # specificly for isaacgym
    if args.sim == "isaacgym" or args.render == "isaacgym":
        from isaacgym import gymapi, gymtorch, gymutil  # noqa: F401

    ## Import put here to support isaacgym
    from metasim.cfg.cameras import PinholeCameraMetaCfg
    from metasim.cfg.scenario import ScenarioMetaCfg
    from metasim.constants import SimType
    from metasim.double_sim import DoubleSimEnv
    from metasim.utils.demo_util import get_traj
    from metasim.utils.setup_util import get_sim_env_class

    camera = PinholeCameraMetaCfg(
        name="camera",
        data_types=["rgb", "depth"],
        width=256,
        height=256,
        pos=(1.5, -1.5, 1.5),  # near
        # pos=(12.0, -12.0, 20.0),  # far
        look_at=(0.0, 0.0, 0.0),
    )
    scenario = ScenarioMetaCfg(
        task=args.task,
        robot=args.robot,
        scene=args.scene,
        cameras=[camera],
        random=args.random,
        render=args.render,
        sim=args.sim,
        renderer=args.renderer,
        num_envs=args.num_envs,
        try_add_table=args.try_add_table,
        object_states=args.object_states,
        split=args.split,
        headless=args.headless,
    )

    num_envs: int = scenario.num_envs

    tic = time.time()
    if scenario.renderer is None:
        log.info(f"Using simulator: {scenario.sim}")
        env_class = get_sim_env_class(SimType(scenario.sim))
        env = env_class(scenario, num_envs, headless=scenario.headless)
    else:
        log.info(f"Using simulator: {scenario.sim}, renderer: {scenario.renderer}")
        env_class_render = get_sim_env_class(SimType(scenario.renderer))
        env_render = env_class_render(scenario, num_envs)  # Isaaclab must launch right after import
        env_class_physics = get_sim_env_class(SimType(scenario.sim))
        env_physics = env_class_physics(scenario, num_envs)  # Isaaclab must launch right after import
        env = DoubleSimEnv(env_physics, env_render)
    toc = time.time()
    log.trace(f"Time to launch: {toc - tic:.2f}s")

    ## Data
    tic = time.time()
    assert os.path.exists(scenario.task.traj_filepath), (
        f"Trajectory file: {scenario.task.traj_filepath} does not exist."
    )
    init_states, all_actions, all_states = get_traj(scenario.task, scenario.robot, env.handler)
    toc = time.time()
    log.trace(f"Time to load data: {toc - tic:.2f}s")

    ########################################################
    ## Main
    ########################################################

    obs_saver = ObsSaver(image_dir=args.save_image_dir, video_path=args.save_video_path)

    ## Reset before first step
    tic = time.time()
    obs, extras = env.reset(states=init_states[:num_envs])
    toc = time.time()
    log.trace(f"Time to reset: {toc - tic:.2f}s")
    obs_saver.add(obs)

    ## Main loop
    step = 0
    while True:
        log.debug(f"Step {step}")
        tic = time.time()
        if scenario.object_states:
            ## TODO: merge states replay into env.step function
            if scenario.sim == "isaacgym":
                raise NotImplementedError("IsaacGym does not support object states")
            if all_states is None:
                raise ValueError("All states are None, please check the trajectory file")
            states = get_states(all_states, step, num_envs)
            env.handler.set_states(states)
            env.handler.refresh_render()
            obs = env.handler.get_observation()

            ## XXX: hack
            success = env.handler.task.checker.check(env.handler)
            if success.any():
                log.info(f"Env {success.nonzero().squeeze(-1).tolist()} succeeded!")
            if success.all():
                break

        else:
            actions = get_actions(all_actions, step, num_envs)
            obs, reward, success, time_out, extras = env.step(actions)

            if success.any():
                log.info(f"Env {success.nonzero().squeeze(-1).tolist()} succeeded!")

            if time_out.any():
                log.info(f"Env {time_out.nonzero().squeeze(-1).tolist()} timed out!")

            if success.all() or time_out.all():
                break

        toc = time.time()
        log.trace(f"Time to step: {toc - tic:.2f}s")

        tic = time.time()
        obs_saver.add(obs)
        toc = time.time()
        log.trace(f"Time to save obs: {toc - tic:.2f}s")
        step += 1

        if args.stop_on_runout and get_runout(all_actions, step):
            log.info("Run out of actions, stopping")
            break

    obs_saver.save()


if __name__ == "__main__":
    main()



from __future__ import annotations

import argparse
import os
import sys
import time

import numpy as np
import rootutils
from curobo.types.math import Pose
from loguru import logger as log
from rich.logging import RichHandler

rootutils.setup_root(__file__, pythonpath=True)
log.configure(handlers=[{"sink": RichHandler(), "format": "{message}"}])
from pytorch3d import transforms

from metasim.cfg.scenario import ScenarioMetaCfg
from metasim.utils.kinematics_utils import get_curobo_models
from metasim.utils.teleop_utils import (
    TRANSFORMATION_MATRIX,
    PhoneServer,
    quaternion_to_rotation_matrix,
    transform_orientation,
)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--task", type=str, required=True)
    parser.add_argument("--robot", type=str, default="franka")
    parser.add_argument("--num_envs", type=int, default=1, help="Number of environments to simulate.")
    parser.add_argument(
        "--sim",
        type=str,
        default="isaaclab",
        choices=["isaaclab", "isaacgym", "pyrep", "pybullet", "sapien", "mujoco"],
    )
    parser.add_argument(
        "--render",
        type=str,
        choices=["isaaclab", "isaacgym", "pyrep", "pybullet", "sapien", "mujoco"],
    )
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    num_envs: int = args.num_envs

    # specificly for isaacgym
    if args.sim == "isaacgym":
        from isaacgym import gymapi, gymtorch, gymutil  # noqa: F401

    ## Import put here to support isaacgym

    import torch

    device = "cuda:0"

    from metasim.cfg.cameras import PinholeCameraMetaCfg
    from metasim.constants import SimType
    from metasim.double_sim import DoubleSimEnv
    from metasim.utils.demo_util import get_traj
    from metasim.utils.setup_util import get_robot, get_sim_env_class, get_task

    task = get_task(args.task)
    robot = get_robot(args.robot)
    camera = PinholeCameraMetaCfg(
        name="camera",
        data_types=["rgb", "depth"],
        width=256,
        height=256,
        pos=(1.5, 0.0, 1.5),
        look_at=(0.0, 0.0, 0.0),
    )
    scenario = ScenarioMetaCfg(task=task, robot=robot, cameras=[camera])

    tic = time.time()
    if args.render is None:
        env_class = get_sim_env_class(SimType(args.sim))
        env = env_class(scenario, num_envs)
    else:
        # print(args.sim, args.render)
        env_class_render = get_sim_env_class(SimType(args.render))
        env_render = env_class_render(scenario, num_envs)
        env_class_physics = get_sim_env_class(SimType(args.sim))
        env_physics = env_class_physics(scenario, num_envs)
        env = DoubleSimEnv(env_physics, env_render)
    toc = time.time()
    log.trace(f"Time to launch: {toc - tic:.2f}s")

    # data
    tic = time.time()
    assert os.path.exists(task.traj_filepath), f"Trajectory file: {task.traj_filepath} does not exist."
    init_states, all_actions, all_states = get_traj(task, robot, env.handler)
    toc = time.time()
    log.trace(f"Time to load data: {toc - tic:.2f}s")

    # reset before first step
    tic = time.time()
    obs, extras = env.reset(states=init_states[:num_envs])
    toc = time.time()
    log.trace(f"Time to reset: {toc - tic:.2f}s")

    # cuRobo IKSysFont()
    *_, robot_ik = get_curobo_models(robot)
    curobo_n_dof = len(robot_ik.robot_config.cspace.joint_names)
    ee_n_dof = len(robot.gripper_release_q)

    sensor_server = PhoneServer(translation_step=0.01, host="0.0.0.0", port=8765, update_dt=1 / 30)
    sensor_server.start_server()

    obs = env.handler.get_observation()
    robot_ee_state = obs["robot_ee_state"].cuda()

    gripper_actuate_tensor = torch.tensor(robot.gripper_actuate_q, dtype=torch.float32, device=device)
    gripper_release_tensor = torch.tensor(robot.gripper_release_q, dtype=torch.float32, device=device)

    # todo: add mobile phone teleopration instructions

    step = 0
    running = True
    while running:
        if not running:
            break

        q_world, delta_posi, gripper_flag = sensor_server.get_latest_data()
        q_world_new = transform_orientation(q_world)

        obs = env.handler.get_observation()

        # compute target
        curr_robot_q = obs["joint_qpos"].cuda()
        robot_ee_state = obs["robot_ee_state"].cuda()
        robot_root_state = obs["robot_root_state"].cuda()
        robot_pos, robot_quat = robot_root_state[:, 0:3], robot_root_state[:, 3:7]
        curr_ee_pos, curr_ee_quat = robot_ee_state[:, 0:3], robot_ee_state[:, 3:7]

        curr_ee_pos = transforms.quaternion_apply(transforms.quaternion_invert(robot_quat), curr_ee_pos - robot_pos)
        curr_ee_quat_local = transforms.quaternion_multiply(transforms.quaternion_invert(robot_quat), curr_ee_quat)

        ee_to_world_matpose = np.eye(4)
        ee_to_world_matpose[:3, :3] = quaternion_to_rotation_matrix(
            transform_orientation(curr_ee_quat[0].cpu().numpy().copy())
        )
        ee_to_world_matpose[:3, 3] = curr_ee_pos.cpu().numpy().copy()
        goal_to_ee_matpose = np.eye(4)
        goal_to_ee_matpose[:3, :3] = TRANSFORMATION_MATRIX
        goal_to_ee_matpose[:3, 3] = delta_posi
        goal = np.dot(ee_to_world_matpose, goal_to_ee_matpose)
        ee_pos_target = goal[:3, 3]
        ee_pos_target_tensor = torch.tensor(ee_pos_target, dtype=torch.float32, device=device)

        close_gripper = gripper_flag
        gripper_q = gripper_actuate_tensor if close_gripper else gripper_release_tensor
        ee_quat_target_local_tensor = torch.tensor(q_world_new, dtype=torch.float32, device=device)

        seed_config = curr_robot_q[:, :curobo_n_dof].unsqueeze(1).tile([1, robot_ik._num_seeds, 1])
        result = robot_ik.solve_batch(
            Pose(ee_pos_target_tensor.unsqueeze(0), ee_quat_target_local_tensor.unsqueeze(0)), seed_config=seed_config
        )

        ik_succ = result.success.squeeze(1)
        q = curr_robot_q.clone()  # shape: [num_envs, robot.num_joints]
        q[ik_succ, :curobo_n_dof] = result.solution[ik_succ, 0].clone()
        q[:, -ee_n_dof:] = gripper_q

        # XXX: this may not work for all simulators, since the order of joints may be different
        actions = [
            {"dof_pos_target": dict(zip(robot.joint_limits.keys(), q[i_env].tolist()))} for i_env in range(num_envs)
        ]
        env.handler.step(actions)
        env.handler.render()

        step += 1
        log.debug(f"Step {step}")

    env.handler.close()
    sys.exit()


if __name__ == "__main__":
    main()



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotCapRightMarker980MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="marker",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/040_large_marker_google_16k/040_large_marker_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/040_cap_right/20240826-215540_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapRightMarker981MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="marker",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/040_large_marker_google_16k/040_large_marker_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/040_cap_right/20240826-222014_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapRightMarker982MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="marker",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/040_large_marker_google_16k/040_large_marker_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/040_cap_right/20240826-221055_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from metasim.cfg.checkers import BaseChecker
from metasim.cfg.objects import NonConvexRigidObjMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, TaskType
from metasim.utils import configclass

traj_filepath_list = [
    "roboverse_data/trajs/simpler_env/GraspSingleOpenedCokeCanInScene-v0/trace_1737906479.0466104_v2.pkl",
    "roboverse_data/trajs/simpler_env/GraspSingleOpenedCokeCanInScene-v0/trace_1737906931.4694195_v2.pkl",
    "roboverse_data/trajs/simpler_env/GraspSingleOpenedCokeCanInScene-v0/trace_1737907890.1946998_v2.pkl",
    "roboverse_data/trajs/simpler_env/GraspSingleOpenedCokeCanInScene-v0/trace_1737908281.1093962_v2.pkl",
]


@configclass
class SimplerEnvGraspOpenedCokeCanMetaCfg(BaseTaskMetaCfg):
    # source_benchmark = BenchmarkType.SIMPLERENVGRASPSINGLEOPENEDCOKECAN
    # task_type = TaskType.TABLETOP_MANIPULATION

    # episode_length = 200

    # objects = [
    #     RigidObjMetaCfg(
    #         name="opened_coke_can", urdf_path="assets/simpler_env/models/coke_can/mobility.urdf", fix_base_link=False
    #     ),
    #     NonConvexRigidObjMetaCfg(
    #         name="scene",
    #         filepath="assets/simpler_env/scenes/google_pick_coke_can_1_v4/google_pick_coke_can_1_v4.glb",
    #         urdf_path="assets/simpler_env/scenes/google_pick_coke_can_1_v4/mobility.urdf",
    #         fix_base_link=True,
    #         mesh_pose=[0, 0, 0, 0.707, 0.707, 0, 0],
    #     ),
    # ]
    # traj_filepath = MISSING
    # checker = BaseChecker()

    def __init__(self, subtask_id=0):
        self.source_benchmark = BenchmarkType.SIMPLERENVGRASPSINGLEOPENEDCOKECAN
        self.task_type = TaskType.TABLETOP_MANIPULATION
        self.episode_length = 200
        self.objects = [
            RigidObjMetaCfg(
                name="opened_coke_can",
                urdf_path="roboverse_data/assets/simpler_env/models/coke_can/mobility.urdf",
                fix_base_link=False,
            ),
            NonConvexRigidObjMetaCfg(
                name="scene",
                filepath="roboverse_data/assets/simpler_env/scenes/google_pick_coke_can_1_v4/google_pick_coke_can_1_v4.glb",
                urdf_path="roboverse_data/assets/simpler_env/scenes/google_pick_coke_can_1_v4/mobility.urdf",
                fix_base_link=True,
                mesh_pose=[0, 0, 0, 0.707, 0.707, 0, 0],
            ),
        ]
        self.traj_filepath = traj_filepath_list[subtask_id]
        self.checker = BaseChecker()



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotCardForthTextualCreditCard253MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-213126_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualCreditCard254MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-211712_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualCreditCard255MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-212554_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualCreditCard256MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-213329_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualCreditCard257MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-212959_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualCreditCard258MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="credit card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/5de830b2cccf4fe7a2e6b400abf26ca7/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/5de830b2cccf4fe7a2e6b400abf26ca7_card_forth_textual/20240826-212908_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1305MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-220111_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1306MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-214929_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1307MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-222638_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1308MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-223055_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1309MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-212455_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCardForthTextualSdCard1310MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="SD card",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/02f7f045679d402da9b9f280030821d4/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/02f7f045679d402da9b9f280030821d4_card_forth_textual/20240826-214843_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from __future__ import annotations

import math
from dataclasses import MISSING
from typing import Literal

from metasim.utils.configclass import configclass


@configclass
class PinholeCameraMetaCfg:
    name: str = MISSING
    data_types: list[Literal["rgb", "depth"]] = MISSING
    width: int = MISSING
    height: int = MISSING
    pos: tuple[float, float, float] = MISSING
    look_at: tuple[float, float, float] = MISSING

    ## Pinhole Camera
    focal_length: float = 24  # same as Isaac Sim 2023.1 version, same as IsaacLab default
    focus_distance: float = 400.0  # same as Isaac Sim 2023.1 version, same as IsaacLab default
    horizontal_aperture: float = 20.955  # same as Isaac Sim 2023.1 version, same as IsaacLab default
    clipping_range: tuple[float, float] = (0.05, 1e5)  # same as Isaac Sim 2023.1 version

    @property
    def vertical_aperture(self) -> float:
        """vertical aperture"""
        return self.horizontal_aperture * self.height / self.width

    @property
    def horizontal_fov(self) -> float:
        """horizontal field of view, in degrees"""
        return 2 * math.atan(self.horizontal_aperture / (2 * self.focal_length)) / math.pi * 180

    @property
    def vertical_fov(self) -> float:
        """vertical field of view, in degrees"""
        return 2 * math.atan(self.vertical_aperture / (2 * self.focal_length)) / math.pi * 180
